---------------------------------------------------------------------------
--
--  (c) Copyright 2011 Xilinx, Inc. All rights reserved.
--
--  This file contains confidential and proprietary information
--  of Xilinx, Inc. and is protected under U.S. and
--  international copyright and other intellectual property
--  laws.
--
--  DISCLAIMER
--  This disclaimer is not a license and does not grant any
--  rights to the materials distributed herewith. Except as
--  otherwise provided in a valid license issued to you by
--  Xilinx, and to the maximum extent permitted by applicable
--  law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
--  WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
--  AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
--  BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
--  INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
--  (2) Xilinx shall not be liable (whether in contract or tort,
--  including negligence, or under any other theory of
--  liability) for any loss or damage of any kind or nature
--  related to, arising under or in connection with these
--  materials, including for any direct, or any indirect,
--  special, incidental, or consequential loss or damage
--  (including loss of data, profits, goodwill, or any type of
--  loss or damage suffered as a result of any action brought
--  by a third party) even if such damage or loss was
--  reasonably foreseeable or Xilinx had been advised of the
--  possibility of the same.
--
--  CRITICAL APPLICATIONS
--  Xilinx products are not designed or intended to be fail-
--  safe, or for use in any application requiring fail-safe
--  performance, such as life-support or safety devices or
--  systems, Class III medical devices, nuclear facilities,
--  applications related to the deployment of airbags, or any
--  other applications that could lead to death, personal
--  injury, or severe property or environmental damage
--  (individually and collectively, "Critical
--  Applications"). Customer assumes the sole risk and
--  liability of any use of Xilinx products in Critical
--  Applications, subject only to applicable laws and
--  regulations governing limitations on product liability.
--
--  THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
--  PART OF THIS FILE AT ALL TIMES.
--
---------------------------------------------------------------------------
-- Description:
-- This is an example testbench for the Complex Multiplier
-- LogiCORE module.  The testbench has been generated by the Xilinx
-- CORE Generator software to accompany the netlist you have generated.
--
-- This testbench is for demonstration purposes only.  See note below for
-- instructions on how to use it with the netlist created for your core.
--
-- See the Complex Multiplier datasheet for further information about
-- this core.
--
---------------------------------------------------------------------------
-- Using this testbench
--
-- This testbench instantiates your generated Complex Multiplier core
-- named "complex_multiplier".
--
-- There are two versions of your core that you can use in this testbench:
-- the XilinxCoreLib behavioral model or the generated netlist.
--
-- 1. XilinxCoreLib behavioral model
--    Compile complex_multiplier.vhd into the work library.  See your
--    simulator documentation for more information on how to do this.
--
-- 2. Generated netlist
--    Execute the following command in the directory containing your CORE
--    Generator output files, to create a VHDL netlist:
--
--      netgen -sim -ofmt vhdl complex_multiplier.ngc complex_multiplier_netlist.vhd
--
--    Compile complex_multiplier_netlist.vhd into the work library.  See your
--    simulator documentation for more information on how to do this.
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

entity tb_complex_multiplier is
end tb_complex_multiplier;

architecture tb of tb_complex_multiplier is

  -----------------------------------------------------------------------
  -- Timing constants
  -----------------------------------------------------------------------
  constant CLOCK_PERIOD : time := 100 ns;
  constant T_HOLD       : time := 10 ns;
  constant T_STROBE     : time := CLOCK_PERIOD - (1 ns);
  constant TEST_CYCLES  : integer := 3000;
  constant PHASE_CYCLES : integer := 1000;

  -----------------------------------------------------------------------
  -- DUT input signals
  -----------------------------------------------------------------------

  -- General inputs
  signal aclk               : std_logic := '0';  -- the master clock
  signal aclken             : std_logic := '1';  -- clock enable

  -- Slave channel A inputs
  signal s_axis_a_tvalid    : std_logic := '0';  -- TVALID for channel A
  signal s_axis_a_tdata     : std_logic_vector(47 downto 0) := (others => 'X');  -- TDATA for channel A

  -- Slave channel B inputs
  signal s_axis_b_tvalid    : std_logic := '0';  -- TVALID for channel B
  signal s_axis_b_tdata     : std_logic_vector(47 downto 0) := (others => 'X');  -- TDATA for channel B

  -- Master channel DOUT inputs
  signal m_axis_dout_tready : std_logic := '0';  -- TREADY for channel DOUT

  -----------------------------------------------------------------------
  -- DUT output signals
  -----------------------------------------------------------------------

  -- Slave channels outputs
  signal s_axis_a_tready    : std_logic := '0';  -- TREADY for channel A
  signal s_axis_b_tready    : std_logic := '0';  -- TREADY for channel B

  -- Master channel DOUT outputs
  signal m_axis_dout_tvalid : std_logic := '0';  -- TVALID for channel DOUT
  signal m_axis_dout_tdata  : std_logic_vector(79 downto 0) := (others => '0');  -- TDATA for channel DOUT

  -----------------------------------------------------------------------
  -- Aliases for AXI channel TDATA fields
  -- These are a convenience for viewing data in a simulator waveform viewer.
  -- If using ModelSim or Questa, add "-voptargs=+acc=n" to the vsim command
  -- to prevent the simulator optimizing away these signals.
  -----------------------------------------------------------------------
  signal s_axis_a_tdata_real     : std_logic_vector(17 downto 0) := (others => '0');
  signal s_axis_a_tdata_imag     : std_logic_vector(17 downto 0) := (others => '0');
  signal s_axis_b_tdata_real     : std_logic_vector(17 downto 0) := (others => '0');
  signal s_axis_b_tdata_imag     : std_logic_vector(17 downto 0) := (others => '0');
  signal m_axis_dout_tdata_real  : std_logic_vector(36 downto 0) := (others => '0');
  signal m_axis_dout_tdata_imag  : std_logic_vector(36 downto 0) := (others => '0');

  -----------------------------------------------------------------------
  -- Testbench signals
  -----------------------------------------------------------------------

  signal cycles       : integer   := 0;    -- Clock cycle counter

  -----------------------------------------------------------------------
  -- Constants, types and functions to create input data
  -- Treat the complex multiplier as a mixer that is combining two
  -- sinusoids exp(+/-jwt) of different frequencies and amplitudes:
  --   channel A: exp(+jwt), frequency = clock / 30, amplitude = fullscale >> 3
  --   channel B: exp(-jwt), frequency = clock / 32, amplitude = fullscale
  -- The output of the complex multiplier is a sinusoid with a frequency
  -- equal to the difference between the input frequencies.
  -----------------------------------------------------------------------

  constant IP_A_DEPTH : integer := 30;
  constant IP_A_WIDTH : integer := 18;
  constant IP_A_SHIFT : integer := 3;  -- bit shift for amplitude
  constant IP_B_DEPTH : integer := 32;
  constant IP_B_WIDTH : integer := 18;
  constant IP_B_SHIFT : integer := 0;  -- no bit shift, max amplitude
  type T_IP_INT_ENTRY is record
    re : integer;
    im : integer;
  end record;
  type T_IP_A_ENTRY is record
    re : std_logic_vector(IP_A_WIDTH-1 downto 0);
    im : std_logic_vector(IP_A_WIDTH-1 downto 0);
  end record;
  type T_IP_B_ENTRY is record
    re : std_logic_vector(IP_B_WIDTH-1 downto 0);
    im : std_logic_vector(IP_B_WIDTH-1 downto 0);
  end record;
  type T_IP_A_TABLE is array (0 to IP_A_DEPTH-1) of T_IP_A_ENTRY;
  type T_IP_B_TABLE is array (0 to IP_B_DEPTH-1) of T_IP_B_ENTRY;

  -- Common function to calculate sine and cosine values
  function create_ip_entry(index, depth, width : integer) return T_IP_INT_ENTRY is
    variable result : T_IP_INT_ENTRY;
    variable theta : real;
  begin
    theta := real(index) / real(depth) * 2.0 * MATH_PI;
    result.re := integer(round(cos(theta) * real(2**(width-2))));
    result.im := integer(round(sin(theta) * real(2**(width-2))));
    return result;
  end function create_ip_entry;

  -- Use separate functions to calculate channel A and B sinusoids as they return different types
  function create_ip_a_table return T_IP_A_TABLE is
    variable result : T_IP_A_TABLE;
    variable entry_int : T_IP_INT_ENTRY;
  begin
    for i in 0 to IP_A_DEPTH-1 loop
      entry_int := create_ip_entry(i, IP_A_DEPTH, IP_A_WIDTH - IP_A_SHIFT);
      result(i).re := std_logic_vector(to_signed(entry_int.re, IP_A_WIDTH));
      result(i).im := std_logic_vector(to_signed(entry_int.im, IP_A_WIDTH));
    end loop;
    return result;
  end function create_ip_a_table;

  function create_ip_b_table return T_IP_B_TABLE is
    variable result : T_IP_B_TABLE;
    variable entry_int : T_IP_INT_ENTRY;
  begin
    for i in 0 to IP_B_DEPTH-1 loop
      entry_int := create_ip_entry(IP_B_DEPTH-1-i, IP_B_DEPTH, IP_B_WIDTH - IP_B_SHIFT);  -- note rotation direction
      result(i).re := std_logic_vector(to_signed(entry_int.re, IP_B_WIDTH));
      result(i).im := std_logic_vector(to_signed(entry_int.im, IP_B_WIDTH));
    end loop;
    return result;
  end function create_ip_b_table;

  -- Call the functions to create the data
  constant IP_A_DATA : T_IP_A_TABLE := create_ip_a_table;
  constant IP_B_DATA : T_IP_B_TABLE := create_ip_b_table;


begin

  -----------------------------------------------------------------------
  -- Instantiate the DUT
  -----------------------------------------------------------------------

  dut : entity work.complex_multiplier
    port map (
      aclk                => aclk,
      aclken              => aclken,
      s_axis_a_tvalid     => s_axis_a_tvalid,
      s_axis_a_tready     => s_axis_a_tready,
      s_axis_a_tdata      => s_axis_a_tdata,
      s_axis_b_tvalid     => s_axis_b_tvalid,
      s_axis_b_tready     => s_axis_b_tready,
      s_axis_b_tdata      => s_axis_b_tdata,
      m_axis_dout_tvalid  => m_axis_dout_tvalid,
      m_axis_dout_tready  => m_axis_dout_tready,
      m_axis_dout_tdata   => m_axis_dout_tdata
      );

  -----------------------------------------------------------------------
  -- Generate clock
  -----------------------------------------------------------------------

  clock_gen : process
  begin
    aclk <= '0';
    wait for CLOCK_PERIOD;
    loop
      cycles <= cycles + 1;
      aclk <= '0';
      wait for CLOCK_PERIOD/2;
      aclk <= '1';
      wait for CLOCK_PERIOD/2;
      if cycles >= TEST_CYCLES then
        report "Not a real failure. Simulation finished successfully." severity failure;
        wait;
      end if;
    end loop;
  end process clock_gen;

  -----------------------------------------------------------------------
  -- Generate clock enable
  -----------------------------------------------------------------------

  -- Disable the clock for 20 clock cycles starting in cycle 100.
  -- Keep the clock enable tied high for the rest of the test.
  aclken_gen : process
  begin
    aclken <= '1';
    -- Drive clock enable T_HOLD time after rising edge of clock
    wait until rising_edge(aclk);
    wait for T_HOLD;
    -- Clock enable goes low in cycle 100, goes high 20 cycles later
    wait for CLOCK_PERIOD * 100;
    aclken <= '0';
    wait for CLOCK_PERIOD * 20;
    aclken <= '1';
    wait;
  end process aclken_gen;

  -----------------------------------------------------------------------
  -- Generate inputs
  -----------------------------------------------------------------------

  stimuli : process
    variable ip_a_index       : integer   := 0;
    variable ip_b_index       : integer   := 0;
    variable a_tvalid_nxt     : std_logic := '0';
    variable b_tvalid_nxt     : std_logic := '0';
    variable dout_tready_nxt  : std_logic := '0';
    variable phase2_cycles : integer := 1;
    variable phase2_count  : integer := 0;
    constant PHASE2_LIMIT  : integer := 30;
    variable phase3_cycles : integer := 1;
    variable phase3_count  : integer := 0;
    constant PHASE3_LIMIT  : integer := 30;
  begin

    -- Test is stopped in clock_gen process, use endless loop here
    loop

      -- Drive inputs T_HOLD time after rising edge of clock
      wait until rising_edge(aclk) and aclken = '1';
      wait for T_HOLD;

      -- Drive AXI handshake signals to demonstrate different types of operation
      case cycles is  -- do different types of operation at different phases of the test
        when 0 to PHASE_CYCLES * 1 - 1 =>
          -- Phase 1: full throughput, no backpressure
          a_tvalid_nxt    := '1';
          b_tvalid_nxt    := '1';
          dout_tready_nxt := '1';
        when PHASE_CYCLES * 1 to PHASE_CYCLES * 2 - 1 =>
          -- Phase 2: apply increasing amounts of backpressure
          a_tvalid_nxt    := '1';
          b_tvalid_nxt    := '1';
          if phase2_count < phase2_cycles then
            dout_tready_nxt := '0';
          else
            dout_tready_nxt := '1';
          end if;
          phase2_count := phase2_count + 1;
          if phase2_count = PHASE2_LIMIT then
            phase2_count  := 0;
            phase2_cycles := phase2_cycles + 1;
          end if;
        when PHASE_CYCLES * 2 to PHASE_CYCLES * 3 - 1 =>
          -- Phase 3: deprive channel A of valid transactions at an increasing rate
          b_tvalid_nxt    := '1';
          dout_tready_nxt := '1';
          if phase3_count < phase3_cycles then
            -- AXI protocol forbids changing TVALID from high to low if TREADY is low
            if s_axis_a_tvalid = '0' or s_axis_a_tready = '1' then
              a_tvalid_nxt := '0';
            end if;
          else
            a_tvalid_nxt := '1';
          end if;
          if phase3_count >= phase3_cycles or s_axis_a_tvalid = '0' or s_axis_a_tready = '1' then
            phase3_count := phase3_count + 1;
            if phase3_count >= PHASE3_LIMIT then
              phase3_count  := 0;
              phase3_cycles := phase3_cycles + 1;
            end if;
          end if;
        when others =>
          -- Test will stop imminently - do nothing
          null;
      end case;

      -- Drive handshake signals with local variable values
      s_axis_a_tvalid <= a_tvalid_nxt;
      s_axis_b_tvalid <= b_tvalid_nxt;
      m_axis_dout_tready <= dout_tready_nxt;

      -- Drive AXI slave channel A payload
      -- Drive 'X's on payload signals when not valid
      -- Payload only changes when TVALID goes high or when a transaction just occurred
      if a_tvalid_nxt /= '1' then
        s_axis_a_tdata <= (others => 'X');
      elsif s_axis_a_tvalid /= '1' or (s_axis_a_tvalid = '1' and s_axis_a_tready = '1') then
        -- TDATA: Real and imaginary components are each 18 bits wide and byte-aligned at their LSBs
        s_axis_a_tdata(17 downto 0) <= IP_A_DATA(ip_a_index).re;
        s_axis_a_tdata(23 downto 18) <= (others => IP_A_DATA(ip_a_index).re(17));  -- sign-extend;
        s_axis_a_tdata(41 downto 24) <= IP_A_DATA(ip_a_index).im;
        s_axis_a_tdata(47 downto 42) <= (others => IP_A_DATA(ip_a_index).im(17));  -- sign-extend;
      end if;

      -- Drive AXI slave channel B payload
      -- Drive 'X's on payload signals when not valid
      -- Payload only changes when TVALID goes high or when a transaction just occurred
      if b_tvalid_nxt /= '1' then
        s_axis_b_tdata <= (others => 'X');
      elsif s_axis_b_tvalid /= '1' or (s_axis_b_tvalid = '1' and s_axis_b_tready = '1') then
        -- TDATA: Real and imaginary components are each 18 bits wide and byte-aligned at their LSBs
        s_axis_b_tdata(17 downto 0) <= IP_B_DATA(ip_b_index).re;
        s_axis_b_tdata(23 downto 18) <= (others => IP_B_DATA(ip_b_index).re(17));  -- sign-extend
        s_axis_b_tdata(41 downto 24) <= IP_B_DATA(ip_b_index).im;
        s_axis_b_tdata(47 downto 42) <= (others => IP_B_DATA(ip_b_index).im(17));  -- sign-extend
      end if;

      -- Increment input data indices
      if a_tvalid_nxt = '1' and s_axis_a_tready = '1' then
        ip_a_index := ip_a_index + 1;
        if ip_a_index = IP_A_DEPTH then
          ip_a_index := 0;
        end if;
      end if;
      if b_tvalid_nxt = '1' and s_axis_b_tready = '1' then
        ip_b_index := ip_b_index + 1;
        if ip_b_index = IP_B_DEPTH then
          ip_b_index := 0;
        end if;
      end if;

    end loop;

  end process stimuli;

  -----------------------------------------------------------------------
  -- Check outputs
  -----------------------------------------------------------------------

  check_outputs : process
    variable check_ok : boolean := true;
    -- Previous values of DOUT channel signals
    variable dout_tvalid_prev : std_logic := '0';
    variable dout_tready_prev : std_logic := '0';
    variable dout_tdata_prev  : std_logic_vector(79 downto 0) := (others => '0');
  begin

    -- Check outputs T_STROBE time after rising edge of clock
    wait until rising_edge(aclk);
    wait for T_STROBE;

    -- Do not check the output payload values, as this requires the behavioral model
    -- which would make this demonstration testbench unwieldy.
    -- Instead, check the protocol of the DOUT channel:
    -- check that the payload is valid (not X) when TVALID is high
    -- and check that the payload does not change while TVALID is high until TREADY goes high

    if m_axis_dout_tvalid = '1' and aclken = '1' then
      if is_x(m_axis_dout_tdata) then
        report "ERROR: m_axis_dout_tdata is invalid when m_axis_dout_tvalid is high" severity error;
        check_ok := false;
      end if;

      if dout_tvalid_prev = '1' and dout_tready_prev = '0' then  -- payload must be the same as last cycle
        if m_axis_dout_tdata /= dout_tdata_prev then
          report "ERROR: m_axis_dout_tdata changed while m_axis_dout_tvalid was high and m_axis_dout_tready was low" severity error;
          check_ok := false;
        end if;
      end if;

    end if;

    assert check_ok
      report "ERROR: terminating test with failures." severity failure;

    -- Record payload values for checking next clock cycle
    if check_ok and aclken = '1' then
      dout_tvalid_prev := m_axis_dout_tvalid;
      dout_tready_prev := m_axis_dout_tready;
      dout_tdata_prev  := m_axis_dout_tdata;
    end if;

  end process check_outputs;

  -----------------------------------------------------------------------
  -- Assign TDATA fields to aliases, for easy simulator waveform viewing
  -----------------------------------------------------------------------

  s_axis_a_tdata_real     <= s_axis_a_tdata(17 downto 0);
  s_axis_a_tdata_imag     <= s_axis_a_tdata(41 downto 24);
  s_axis_b_tdata_real     <= s_axis_b_tdata(17 downto 0);
  s_axis_b_tdata_imag     <= s_axis_b_tdata(41 downto 24);
  m_axis_dout_tdata_real  <= m_axis_dout_tdata(36 downto 0);
  m_axis_dout_tdata_imag  <= m_axis_dout_tdata(76 downto 40);

end tb;

