----------------------------------------------------------------------------------
--    _____
--   /     \
--  /____   \____
-- / \===\   \==/
--/___\===\___\/  AVNET
--     \======/
--      \====/    
--
----------------------------------------------------------------------------------
-- This design is the property of Avnet.  Publication of this
-- design is not authorized without written consent from Avnet.
-- 
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Disclaimer:
--    Avnet, Inc. makes no warranty for the use of this code or design.
--    This code is provided  "As Is". Avnet, Inc assumes no responsibility for
--    any errors, which may appear in this code, nor does it make a commitment
--    to update the information contained herein. Avnet, Inc specifically
--    disclaims any implied warranties of fitness for a particular purpose.
--                     Copyright(c) 2010 Avnet, Inc.
--                             All rights reserved.
--
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
----------------------------------------------------------------------------------
--
----------------------------------------------------------------------------------
-- FILE NAME : ML605_fmc150.vhd
--
-- AUTHOR    : Luc Langlois
--
-- COMPANY   : Avnet
--
-- UNITS     : Entity       - ML605_fmc150
--             architecture - ML605_fmc150_syn
--
-- LANGUAGE  : VHDL
--
-- Target Device: V6LX240T-2FF1156
-- Tool versions: ISE Design Suite: System Edition 13.1 / build O.40d
--
-------------------------------------------------------------------------------------
--
-------------------------------------------------------------------------------------
-- DESCRIPTION
-- ===========

-------------------------------------------------------------------------------------
-- Library declarations
-------------------------------------------------------------------------------------
library ieee;
  use ieee.std_logic_unsigned.all;
  use ieee.std_logic_misc.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_1164.all;
library unisim;
  use unisim.vcomponents.all;

-------------------------------------------------------------------------------------
-- Entity declaration
-------------------------------------------------------------------------------------
entity ML605_fmc150 is
generic (
  MAX_PATTERN_CNT : integer := 1000 -- value of 15000 = approx 1 sec for ramp of length 2^14 samples @ 245.76 MSPS
);
port (

  --ML605 Resources
  cpu_reset        : in    std_logic; -- CPU RST button, SW10 on ML605
  sysclk_p         : in    std_logic;
  sysclk_n         : in    std_logic;
  gpio_led         : out   std_logic_vector(7 downto 0);
  gpio_dip_sw      : in    std_logic_vector(7 downto 0);
  gpio_led_c       : out   std_logic;
  gpio_led_e       : out   std_logic;
  gpio_led_n       : out   std_logic;
  gpio_led_s       : out   std_logic;
  gpio_led_w       : out   std_logic;
  gpio_sw_c        : in    std_logic;
  gpio_sw_e        : in    std_logic;
  gpio_sw_n        : in    std_logic;
  gpio_sw_s        : in    std_logic;
  gpio_sw_w        : in    std_logic;

  --Clock/Data connection to ADC on FMC150 (ADS62P49)
  clk_ab_p         : in    std_logic;
  clk_ab_n         : in    std_logic;
  cha_p            : in    std_logic_vector(6 downto 0);
  cha_n            : in    std_logic_vector(6 downto 0);
  chb_p            : in    std_logic_vector(6 downto 0);
  chb_n            : in    std_logic_vector(6 downto 0);

  --Clock/Data connection to DAC on FMC150 (DAC3283)
  dac_dclk_p       : out   std_logic;
  dac_dclk_n       : out   std_logic;
  dac_data_p       : out   std_logic_vector(7 downto 0);
  dac_data_n       : out   std_logic_vector(7 downto 0);
  dac_frame_p      : out   std_logic;
  dac_frame_n      : out   std_logic;
  txenable         : out   std_logic;

  --Clock/Trigger connection to FMC150
  clk_to_fpga_p    : in    std_logic;
  clk_to_fpga_n    : in    std_logic;
  ext_trigger_p    : in    std_logic;
  ext_trigger_n    : in    std_logic;

  --Serial Peripheral Interface (SPI)
  spi_sclk         : out   std_logic; -- Shared SPI clock line
  spi_sdata        : out   std_logic; -- Shared SPI sata line

  -- ADC specific signals
  adc_n_en         : out   std_logic; -- SPI chip select
  adc_sdo          : in    std_logic; -- SPI data out
  adc_reset        : out   std_logic; -- SPI reset

  -- CDCE specific signals
  cdce_n_en        : out   std_logic; -- SPI chip select
  cdce_sdo         : in    std_logic; -- SPI data out
  cdce_n_reset     : out   std_logic;
  cdce_n_pd        : out   std_logic;
  ref_en           : out   std_logic;
  pll_status       : in    std_logic;

  -- DAC specific signals
  dac_n_en         : out   std_logic; -- SPI chip select
  dac_sdo          : in    std_logic; -- SPI data out

  -- Monitoring specific signals
  mon_n_en         : out   std_logic; -- SPI chip select
  mon_sdo          : in    std_logic; -- SPI data out
  mon_n_reset      : out   std_logic;
  mon_n_int        : in    std_logic;

  --FMC Present status
  prsnt_m2c_l      : in    std_logic

);
end ML605_fmc150;

architecture ML605_fmc150_syn of ML605_fmc150 is

----------------------------------------------------------------------------------------------------
-- Constant declaration
----------------------------------------------------------------------------------------------------
constant CLK_IDELAY : integer := 0; -- Initial number of delay taps on ADC clock input
constant CHA_IDELAY : integer := 25; -- Initial number of delay taps on ADC data port A  -- error-free capture range measured between 20 ... 30
constant CHB_IDELAY : integer := 24; -- Initial number of delay taps on ADC data port B  -- error-free capture range measured between 18 ... 30

----------------------------------------------------------------------------------------------------
-- Component declaration
----------------------------------------------------------------------------------------------------
component icon is
port (
  control0 : inout std_logic_vector(35 downto 0);
  control1 : inout std_logic_vector(35 downto 0);
  control2 : inout std_logic_vector(35 downto 0)
);
end component icon;

component vio is
port (
  clk       : in    std_logic := 'X';
  sync_in   : in    std_logic_vector(47 downto 0);
  control   : inout std_logic_vector(35 downto 0);
  sync_out  : out   std_logic_vector(67 downto 0)
);
end component vio;

component ila_dac is
port (
  clk     : in    std_logic := 'X';
  trig0   : in    std_logic_vector(31 downto 0);
  control : inout std_logic_vector(35 downto 0)
);
end component ila_dac;

component mmcm_adac is
port (
  -- Clock in ports
  clk_in1   : in  std_logic;
  clk_in2   : in     std_logic;
  clk_in_sel : in     std_logic;
  -- Clock out ports
  clk_out1  : out std_logic;
  clk_out2  : out std_logic;
  clk_out3  : out std_logic;
  -- Status and control signals
  reset     : in  std_logic;
  locked    : out std_logic
);
end component mmcm_adac;




component mmcm is
port (
  -- Clock in ports
  clk_in1   : in std_logic;
  clk_in2   : in std_logic;
  clk_in_sel  : in std_logic;
  -- Clock out ports
  clk_out1  : out std_logic;
  clk_out2  : out std_logic;
  -- Status and control signals
  reset     : in  std_logic;
  locked    : out std_logic
);
end component mmcm;

component fmc150_spi_ctrl is
port (
  -- VIO command interface
  odata            : out   std_logic_vector(31 downto 0);
  odata_valid      : out   std_logic;
  addr             : in    std_logic_vector(15 downto 0);
  idata            : in    std_logic_vector(31 downto 0);
  rd_n_wr          : in    std_logic;
  cdce72010_valid  : in    std_logic;
  ads62p49_valid   : in    std_logic;
  dac3283_valid    : in    std_logic;
  amc7823_valid    : in    std_logic;
  
  busy             : out   std_logic;
  
  external_clock   : in    std_logic;

  -- Global signals
  rst              : in    std_logic;
  clk              : in    std_logic;

  -- External signals
  spi_sclk         : out   std_logic;
  spi_sdata        : out   std_logic;

  adc_n_en         : out   std_logic;
  adc_sdo          : in    std_logic;
  adc_reset        : out   std_logic;

  cdce_n_en        : out   std_logic;
  cdce_sdo         : in    std_logic;
  cdce_n_reset     : out   std_logic;
  cdce_n_pd        : out   std_logic;
  ref_en           : out   std_logic;
  pll_status       : in    std_logic;

  dac_n_en         : out   std_logic;
  dac_sdo          : in    std_logic;

  mon_n_en         : out   std_logic;
  mon_sdo          : in    std_logic;
  mon_n_reset      : out   std_logic;
  mon_n_int        : in    std_logic;

  prsnt_m2c_l      : in    std_logic;
  init_amc7823_done	 : out   std_logic
);
end component fmc150_spi_ctrl;


component ila_adc_cali
  port (
    control : inout std_logic_vector(35 downto 0);
    clk : in std_logic;
    data : in std_logic_vector(23 downto 0);
    trig0 : in std_logic_vector(13 downto 0);
    trig1 : in std_logic_vector(0 to 0);
    trig2 : in std_logic_vector(0 to 0));
end component;

component ADC_auto_calibration is
  generic (
	  MAX_PATTERN_CNT : integer := 1000;   -- value of 15000 = approx 1 sec for ramp of length 2^14 samples @ 245.76 MSPS
	  INIT_IDELAY : integer                -- Initial number of delay taps on ADC data port
	);
  Port ( 
     reset                 : in  STD_LOGIC;
	  clk                   : in  STD_LOGIC;
	  ADC_calibration_start : in  STD_LOGIC;
	  ADC_data              : in  STD_LOGIC_VECTOR (13 downto 0);
	  re_mux_polarity       : out  STD_LOGIC;
	  trace_edge            : out  STD_LOGIC;
	  ADC_calibration_state : out  STD_LOGIC_VECTOR(2 downto 0);
	  iDelay_cnt            : out  STD_LOGIC_VECTOR (4 downto 0);
	  ADC_calibration_done  : out  BOOLEAN;
	  ADC_calibration_good  : out  STD_LOGIC);
end component;


component sample_pass is port (
clk_fs					: in  std_logic;
clk_gfs					: in  std_logic;
din_a						: in  STD_LOGIC_VECTOR (13 downto 0);
din_b						: in  STD_LOGIC_VECTOR (13 downto 0);
dout_a						: out STD_LOGIC_VECTOR (15 downto 0);
dout_b						: out STD_LOGIC_VECTOR (15 downto 0);
gpio_dip_sw				: in	STD_LOGIC_VECTOR (7 downto 0));

end component;

----------------------------------------------------------------------------------------------------
-- Signal declaration
----------------------------------------------------------------------------------------------------
signal vio_sync_in       : std_logic_vector(47 downto 0);
signal vio_control       : std_logic_vector(35 downto 0);
signal vio_sync_out      : std_logic_vector(67 downto 0);
signal vio_sync_out_prev : std_logic_vector(96 downto 0);
  -- Ensure the tools keep these signals for ChipScope
attribute S: string;
attribute S of vio_sync_in : signal is "TRUE";
attribute S of vio_control : signal is "TRUE";
attribute S of vio_sync_out : signal is "TRUE";
attribute keep: string;
attribute keep of vio_sync_in : signal is "TRUE";
attribute keep of vio_control : signal is "TRUE";
attribute keep of vio_sync_out : signal is "TRUE";

signal clk_100Mhz        : std_logic;
signal clk_200Mhz        : std_logic;
signal mmcm_locked       : std_logic;

signal arst              : std_logic;
signal rst               : std_logic;

signal clk_ab_l          : std_logic;
signal clk_ab_dly        : std_logic;
signal clk_ab_i          : std_logic;

signal cha_ddr           : std_logic_vector(6 downto 0);  -- Double Data Rate
signal cha_ddr_dly       : std_logic_vector(6 downto 0);  -- Double Data Rate, Delayed
signal cha_sdr_sample1   : std_logic_vector(13 downto 0); -- Single Data Rate
signal cha_sdr_sample0   : std_logic_vector(13 downto 0); -- Single Data Rate

signal adc_dout_122_88_MSPS_valid		: std_logic; -- data_valid pulse accompanies re-clocked 122.88 MSPS data at 245.75 MHz towards SysGen GateWays IN
signal cha_sdr_sample1_re_clocked   : std_logic_vector(13 downto 0); -- Single Data Rate @ 122.88 MSPS, re-clocked at 245.75 MHz accompanied by data_valid pulse
signal cha_sdr_sample0_re_clocked   : std_logic_vector(13 downto 0); -- Single Data Rate @ 122.88 MSPS, re-clocked at 245.75 MHz accompanied by data_valid pulse
signal chb_sdr_sample1_re_clocked   : std_logic_vector(13 downto 0); -- Single Data Rate @ 122.88 MSPS, re-clocked at 245.75 MHz accompanied by data_valid pulse
signal chb_sdr_sample0_re_clocked   : std_logic_vector(13 downto 0); -- Single Data Rate @ 122.88 MSPS, re-clocked at 245.75 MHz accompanied by data_valid pulse

signal chb_ddr           : std_logic_vector(6 downto 0);  -- Double Data Rate
signal chb_ddr_dly       : std_logic_vector(6 downto 0);  -- Double Data Rate, Delayed
signal chb_sdr_sample1   : std_logic_vector(13 downto 0); -- Single Data Rate
signal chb_sdr_sample0   : std_logic_vector(13 downto 0); -- Single Data Rate

signal adc_dout_a        : std_logic_vector(13 downto 0);
signal adc_dout_a_prev   : std_logic_vector(13 downto 0);
signal adc_dout_b        : std_logic_vector(13 downto 0);
signal adc_dout_valid	 : std_logic;
signal clk_245_76MHz_count 			 : std_logic;

type 	 states is (WAIT_TO_START, PRE_SEQUENCE_1, SEQUENCE_1, SEQUENCE_1_OK, PRE_SEQUENCE_2, SEQUENCE_2, SEQUENCE_2_OK, ADC_DATA_CAPTURE_BAD);
signal state       		 : states;
signal ADC_test_sequence_good	: std_logic;

signal cmplx_aresetn_ddc : std_logic;

signal delay_update      : std_logic;
signal clk_cntvaluein    : std_logic_vector(4 downto 0);
signal clk_cntvaluein_update    : std_logic_vector(4 downto 0);
signal clk_cntvaluein_update_vio    : std_logic_vector(4 downto 0);
signal cha_cntvaluein    : std_logic_vector(4 downto 0);
signal cha_cntvaluein_update    : std_logic_vector(4 downto 0);
signal cha_cntvaluein_update_vio    : std_logic_vector(4 downto 0);
signal cha_cntvaluein_update_245_76MHz    : std_logic_vector(4 downto 0);
signal cha_cntvaluein_update_100MHz    : std_logic_vector(4 downto 0);
signal chb_cntvaluein    : std_logic_vector(4 downto 0);
signal chb_cntvaluein_update    : std_logic_vector(4 downto 0);
signal chb_cntvaluein_update_vio    : std_logic_vector(4 downto 0);
signal chb_cntvaluein_update_245_76MHz    : std_logic_vector(4 downto 0);
signal chb_cntvaluein_update_100MHz    : std_logic_vector(4 downto 0);

signal clk_cntvalueout   : std_logic_vector(4 downto 0);
type cha_cntvalueout_array is array(cha_p'length-1 downto 0) of std_logic_vector(4 downto 0);
signal cha_cntvalueout   : cha_cntvalueout_array;
type chb_cntvalueout_array is array(chb_p'length-1 downto 0) of std_logic_vector(4 downto 0);
signal chb_cntvalueout   : chb_cntvalueout_array;

signal rd_n_wr           : std_logic;
signal addr              : std_logic_vector(15 downto 0);
signal idata             : std_logic_vector(31 downto 0);
signal odata             : std_logic_vector(31 downto 0);
signal busy              : std_logic;
signal busy_reg	       : std_logic;
signal cdce72010_valid   : std_logic;
signal ads62p49_valid    : std_logic;
signal dac3283_valid     : std_logic;
signal amc7823_valid     : std_logic;
signal fmc150_spi_ctrl_done	: std_logic;
signal fmc150_spi_ctrl_done_r	: std_logic;

signal clk_122_88MHz		 : std_logic;
signal clk_245_76MHz		 : std_logic;
signal clk_491_52MHz	    : std_logic;
signal mmcm_adac_locked  : std_logic;

signal clk_to_fpga       : std_logic;

signal dac_din_a         : std_logic_vector(15 downto 0);
signal dac_din_b         : std_logic_vector(15 downto 0);

signal frame             : std_logic;
signal io_rst          	 : std_logic;

signal dac_dclk_prebuf   : std_logic;
signal dac_data_prebuf   : std_logic_vector(7 downto 0);
signal dac_frame_prebuf  : std_logic;

signal adc_cha_re_mux_polarity 	 : std_logic := '1';	-- initial state '1' is contrary to actual default behaviour in hardware, but desired for simulation to verify correctness of state machine
signal adc_chb_re_mux_polarity 	 : std_logic := '1';	-- initial state '1' is contrary to actual default behaviour in hardware, but desired for simulation to verify correctness of state machine

signal digital_mode      : std_logic;
signal ddc_duc_bypass    : std_logic;
signal adc_out_dac_in    : std_logic;
signal external_clock    : std_logic;
signal duc_dcc_route_ctrl_sig	: std_logic_vector(2 downto 0);

signal ADC_calibration	 : std_logic;
signal ADC_calibration_r	 : std_logic;
signal ADC_calibration_rr	 : std_logic;

signal crap  : std_logic_vector(24 downto 0);

signal baseband_out_i_sig    	: std_logic_vector(15 downto 0);
signal baseband_out_q_sig    	: std_logic_vector(15 downto 0);
signal baseband_out_valid_sig	: std_logic;
signal baseband_out_valid_sig_v	: std_logic_vector(0 downto 0);

signal ila_baseband_out_trig0    : std_logic_vector(31 downto 0);
signal ila_baseband_out_trig1    : std_logic_vector(0 downto 0);
signal ila_baseband_out_control	: std_logic_vector(35 downto 0);

signal ila_dac_trig0     : std_logic_vector(31 downto 0);
signal ila_dac_control   : std_logic_vector(35 downto 0);

signal ADC_cha_calibration_start       : std_logic;
signal ADC_chb_calibration_start       : std_logic;
signal ADC_cha_calibration_done        : boolean;
signal ADC_cha_calibration_done_r      : boolean;
signal ADC_cha_calibration_done_rr     : boolean;
signal ADC_chb_calibration_done        : boolean;
signal ADC_chb_calibration_done_r      : boolean;
signal ADC_chb_calibration_done_rr     : boolean;
signal ADC_chb_calibration_test_pattern_mode_command_sent : boolean;
signal ADC_cha_calibration_test_pattern_mode_command_sent : boolean;
signal ADC_chb_normal_mode_command_sent : boolean;
signal ADC_cha_normal_mode_command_sent : boolean;
signal ADC_chb_trace_edge              : std_logic;
signal ADC_cha_trace_edge              : std_logic;
signal ADC_chb_calibration_state       : std_logic_vector(2 downto 0);
signal ADC_cha_calibration_state       : std_logic_vector(2 downto 0);
signal ADC_chb_calibration_good	      : std_logic;
signal ADC_cha_calibration_good	      : std_logic;
signal ADC_calibration_good	         : std_logic;
signal ADC_chb_ready                   : boolean;
signal ADC_cha_ready                   : boolean;
signal ADC_ready                       : boolean;

signal ila_adc_cali_control	         : std_logic_vector(35 downto 0);
signal ILA_ADC_cali_data 		         : std_logic_vector(23 downto 0);
signal ILA_ADC_cali_inst_trig0         : std_logic_vector(0 downto 0);
signal ILA_ADC_cali_inst_trig1         : std_logic_vector(0 downto 0);
signal ILA_ADC_cali_inst_trig2         : std_logic_vector(0 downto 0);

signal sysclk_buf                      : std_logic;

----------------------------------------------------------------------------------------------------
-- Begin
----------------------------------------------------------------------------------------------------
begin

--------------------------------------------------------------------------------------------------
-- VIO
--------------------------------------------------------------------------------------------------
vio_inst : vio
port map (
  clk       => clk_100Mhz,
  sync_in   => vio_sync_in,
  control   => vio_control,
  sync_out  => vio_sync_out
);

----------------------------------------------------------------------------------
-- VIO In Mapping
----------------------------------------------------------------------------------

vio_sync_in <=
  busy               & -- (47 downto 47)
  odata              & -- (46 downto 15)
  clk_cntvalueout    & -- (14 downto 10)
  chb_cntvalueout(0) & -- (09 downto 05)
  cha_cntvalueout(0);  -- (04 downto 00)

----------------------------------------------------------------------------------
-- VIO Out / or incoming data received from GUI RX FIFO Mapping
----------------------------------------------------------------------------------
routing_to_SPI: process (arst, clk_100Mhz)
begin
	if (arst = '1') then
      busy_reg <= '0';
		cdce72010_valid	<= '0';
		ads62p49_valid		<= '0';
		dac3283_valid		<= '0';
		amc7823_valid		<= '0';
		cha_cntvaluein_update_vio <= vio_sync_out(4 downto 0);
		chb_cntvaluein_update_vio <= vio_sync_out(9 downto 5);
		clk_cntvaluein_update_vio <= vio_sync_out(14 downto 10);
      cha_cntvaluein_update_100MHz <= conv_std_logic_vector(CHA_IDELAY, 5);
		cha_cntvaluein_update <= conv_std_logic_vector(CHA_IDELAY, 5);
		chb_cntvaluein_update_100MHz <= conv_std_logic_vector(CHB_IDELAY, 5);
		chb_cntvaluein_update <= conv_std_logic_vector(CHB_IDELAY, 5);
		clk_cntvaluein_update <= conv_std_logic_vector(CLK_IDELAY, 5);
		ADC_chb_calibration_done_r <= FALSE;
		ADC_chb_calibration_done_rr <= FALSE;
		ADC_cha_calibration_done_r <= FALSE;
		ADC_cha_calibration_done_rr <= FALSE;
		ADC_chb_calibration_test_pattern_mode_command_sent <= FALSE;
      ADC_cha_calibration_test_pattern_mode_command_sent <= FALSE;
		ADC_chb_normal_mode_command_sent <= FALSE;
      ADC_cha_normal_mode_command_sent <= FALSE;
      ADC_chb_calibration_start <= '0';
		ADC_cha_calibration_start <= '0';
		fmc150_spi_ctrl_done_r <= '0';
      ADC_cha_ready <= FALSE;
      ADC_chb_ready <= FALSE;
		ADC_ready <= FALSE;
	elsif (rising_edge(clk_100Mhz)) then
      busy_reg <= busy;
		fmc150_spi_ctrl_done_r <= fmc150_spi_ctrl_done;
		ADC_chb_calibration_done_r <= ADC_chb_calibration_done;									-- double-register to cross from clock domain of 'ADC_auto_calibration'
		ADC_chb_calibration_done_rr <= ADC_chb_calibration_done_r;								-- where 'ADC_chb_calibration_done' is set
		ADC_cha_calibration_done_r <= ADC_cha_calibration_done;
		ADC_cha_calibration_done_rr <= ADC_cha_calibration_done_r;
		if not ADC_chb_ready then
			chb_cntvaluein_update_100MHz <= chb_cntvaluein_update_245_76MHz;
			chb_cntvaluein_update <= chb_cntvaluein_update_100MHz;
			if not ADC_chb_calibration_done_rr then
				if not ADC_chb_calibration_test_pattern_mode_command_sent then 
					if (fmc150_spi_ctrl_done = '1' and fmc150_spi_ctrl_done_r = '0') then	-- rising edge of 'fmc150_spi_ctrl_done' indicates reset-time
																													-- initialization of FMC150 SPI devices has completed
						addr <= x"0075";
						idata <= x"00000004";																-- send SPI command to ads62p49 for test-mode / ramp pattern on Ch B
						rd_n_wr <= '0';
						ads62p49_valid <= not ads62p49_valid;											-- toggle triggers transaction with SPI device on FMC150
						ADC_chb_calibration_test_pattern_mode_command_sent <= TRUE;
					else
						ads62p49_valid <= ads62p49_valid;
						ADC_chb_calibration_test_pattern_mode_command_sent <= FALSE;					
					end if;
				else
					if (busy = '0' and busy_reg = '1') then	-- wait for falling edge of 'busy' indicating SPI port has sent command to ADS62P49 for test-mode
						ADC_chb_calibration_start <= '1'; 			-- ... ADC auto-calibration state-machine 'ADC_auto_calibration' is awaiting this event to start
					else
						ADC_chb_calibration_start <= '0';
					end if;
				end if;
			else
				if not ADC_chb_normal_mode_command_sent then
					addr <= x"0075";
					idata <= x"00000000";																	-- send SPI command to ads62p49 for normal capture mode
					rd_n_wr <= '0';
					ads62p49_valid <= not ads62p49_valid;												-- toggle triggers transaction with SPI device on FMC150
					ADC_chb_normal_mode_command_sent <= TRUE;
				else
					if (busy = '0' and busy_reg = '1') then		-- wait for falling edge of 'busy' indicating SPI port has sent command to ADS62P49 to resume normal capture mode after ADC calibration sequence
						ADC_chb_ready <= TRUE;																	-- ADC auto-calibration is done and ADS62P49 is now in normal capture mode ... allow RX FIFO to read
					else
						ADC_chb_ready <= FALSE;
					end if;
				end if;
			end if;
      elsif not ADC_cha_ready then
			cha_cntvaluein_update_100MHz <= cha_cntvaluein_update_245_76MHz;
			cha_cntvaluein_update <= cha_cntvaluein_update_100MHz;
			if not ADC_cha_calibration_done_rr then
				if not ADC_cha_calibration_test_pattern_mode_command_sent then 
               addr <= x"0062";
					idata <= x"00000004";																-- send SPI command to ads62p49 for test-mode / ramp pattern on Ch A
					rd_n_wr <= '0';
					ads62p49_valid <= not ads62p49_valid;											-- toggle triggers transaction with SPI device on FMC150
					ADC_cha_calibration_test_pattern_mode_command_sent <= TRUE;
				else
					if (busy = '0' and busy_reg = '1') then	-- wait for falling edge of 'busy' indicating SPI port has sent command to ADS62P49 for test-mode
						ADC_cha_calibration_start <= '1'; 			-- ... ADC auto-calibration state-machine 'ADC_auto_calibration' is awaiting this event to start
					else
						ADC_cha_calibration_start <= '0';
					end if;
				end if;
			else
				if not ADC_cha_normal_mode_command_sent then
					addr <= x"0062";
					idata <= x"00000000";																	-- send SPI command to ads62p49 for normal capture mode
					rd_n_wr <= '0';
					ads62p49_valid <= not ads62p49_valid;												-- toggle triggers transaction with SPI device on FMC150
					ADC_cha_normal_mode_command_sent <= TRUE;
				else
					if (busy = '0' and busy_reg = '1') then		-- wait for falling edge of 'busy' indicating SPI port has sent command to ADS62P49 to resume normal capture mode after ADC calibration sequence
						ADC_cha_ready <= TRUE;																-- ADC auto-calibration is done and ADS62P49 is now in normal capture mode
                  ADC_ready <= TRUE;                                                   -- allow RX FIFO to read
					else
						ADC_cha_ready <= FALSE;
                  ADC_ready <= FALSE;
					end if;
				end if;
			end if;
		else																		----------------------- FMC150 SPI registers controlled by VIO
			addr						<= vio_sync_out(30 downto 15);
			idata						<= vio_sync_out(62 downto 31);
			rd_n_wr					<= vio_sync_out(63);
			cdce72010_valid		<= vio_sync_out(64);
			ads62p49_valid			<= vio_sync_out(65);
			dac3283_valid			<= vio_sync_out(66);
			amc7823_valid			<= vio_sync_out(67);
			cha_cntvaluein_update_vio <= vio_sync_out(4 downto 0);
			chb_cntvaluein_update_vio <= vio_sync_out(9 downto 5);
			clk_cntvaluein_update_vio <= vio_sync_out(14 downto 10);
			if (cha_cntvaluein_update_vio /= vio_sync_out(4 downto 0)) then
				cha_cntvaluein_update <= vio_sync_out(4 downto 0);								-- triggers an update at process iDelay_update when user updates iDelay in VIO
			else
				cha_cntvaluein_update <= cha_cntvaluein_update;
			end if;
			if (chb_cntvaluein_update_vio /= vio_sync_out(9 downto 5)) then
				chb_cntvaluein_update <= vio_sync_out(9 downto 5);								-- triggers an update at process iDelay_update when user updates iDelay in VIO
			else
				chb_cntvaluein_update <= chb_cntvaluein_update;
			end if;
			if (clk_cntvaluein_update_vio /= vio_sync_out(14 downto 10)) then
				clk_cntvaluein_update <= vio_sync_out(14 downto 10);							-- triggers an update at process iDelay_update when user updates iDelay in VIO
			else
				clk_cntvaluein_update <= clk_cntvaluein_update;
			end if;
		end if;
	end if;
end process routing_to_SPI;

----------------------------------------------------------------------------------
-- Update iDelay values for incoming ADC data, clock
----------------------------------------------------------------------------------

iDelay_update: process (arst, clk_100Mhz)
begin
  if (arst = '1') then
    delay_update   <= '1';
    clk_cntvaluein <= conv_std_logic_vector(CLK_IDELAY, 5);
    cha_cntvaluein <= conv_std_logic_vector(CHA_IDELAY, 5);
    chb_cntvaluein <= conv_std_logic_vector(CHB_IDELAY, 5);
  elsif (rising_edge(clk_100Mhz)) then
  
    -- Generate an delay_update pulse in case one of the cntvaluein values has changed
    if (cha_cntvaluein /= cha_cntvaluein_update) then
      delay_update   <= '1';
      clk_cntvaluein <= clk_cntvaluein;
      chb_cntvaluein <= chb_cntvaluein;
      cha_cntvaluein <= cha_cntvaluein_update;
    elsif (chb_cntvaluein /= chb_cntvaluein_update) then
      delay_update   <= '1';
      clk_cntvaluein <= clk_cntvaluein;
      chb_cntvaluein <= chb_cntvaluein_update;
      cha_cntvaluein <= cha_cntvaluein;
    elsif (clk_cntvaluein /= clk_cntvaluein_update) then
      delay_update   <= '1';
      clk_cntvaluein <= clk_cntvaluein_update;
      chb_cntvaluein <= chb_cntvaluein;
      cha_cntvaluein <= cha_cntvaluein;
    else
      delay_update   <= '0';
      clk_cntvaluein <= clk_cntvaluein;
      chb_cntvaluein <= chb_cntvaluein;
      cha_cntvaluein <= cha_cntvaluein;
    end if;
  end if;
end process iDelay_update;

-- Differential input buffer
ibufds_inst_sysclk : ibufds
generic map (
  IOSTANDARD => "LVDS_25",
  IBUF_LOW_PWR => FALSE,
  DIFF_TERM  => TRUE
)
port map (
  i  => sysclk_p,
  ib => sysclk_n,
  o  => sysclk_buf
);

----------------------------------------------------------------------------------------------------
-- MMCM System Clock
----------------------------------------------------------------------------------------------------
mmcm_inst : mmcm
port map (
  clk_in1 => sysclk_buf,
  clk_in2 => sysclk_buf,
  clk_in_sel => '0',
  clk_out1  => clk_100Mhz,
  clk_out2  => clk_200Mhz,
  reset     => cpu_reset,
  locked    => mmcm_locked
);

arst <= not mmcm_locked;

----------------------------------------------------------------------------------------------------
-- Clock from ADC on FMC150 for channel A and B
----------------------------------------------------------------------------------------------------

-- Differential input buffer
ibufds_inst : ibufds
generic map (
  IOSTANDARD => "LVDS_25",
  IBUF_LOW_PWR => FALSE,
  DIFF_TERM  => TRUE
)
port map (
  i  => clk_ab_p,
  ib => clk_ab_n,
  o  => clk_ab_l
);

   IDELAYE1_inst_ADC_CLK : IODELAYE1
   generic map (
      CINVCTRL_SEL => FALSE,          -- Enable dynamic clock inversion (TRUE/FALSE)
      DELAY_SRC => "I",               -- Delay input ("I", "CLKIN", "DATAIN", "IO", "O")
      HIGH_PERFORMANCE_MODE => TRUE,  -- Reduced jitter (TRUE), Reduced power (FALSE)
      IDELAY_TYPE => "VAR_LOADABLE",  -- "DEFAULT", "FIXED", "VARIABLE", or "VAR_LOADABLE" 
      IDELAY_VALUE => CLK_IDELAY,     -- Input delay tap setting (0-31)
      ODELAY_TYPE => "FIXED",         -- "FIXED", "VARIABLE", or "VAR_LOADABLE" 
      ODELAY_VALUE => 0,              -- Output delay tap setting (0-31)
      REFCLK_FREQUENCY => 200.00,  	  -- IDELAYCTRL clock input frequency in MHz
      SIGNAL_PATTERN => "CLOCK"       -- "DATA" or "CLOCK" input signal
   )
   port map (
      CNTVALUEOUT => clk_cntvalueout, 		-- 5-bit output: Counter value output	-- NOTE: need clock-domain crossing 
      DATAOUT => clk_ab_dly,         		-- 1-bit output: Delayed data output
      C => clk_122_88MHz,                 -- 1-bit input: Clock input				-- NOTE: Virtex-6 IODELAYE1 imposes same clock source as iSERDES CLKDIV
      CE => '0',                  			-- 1-bit input: Active high enable increment/decrement input
      CINVCTRL => '0',            			-- 1-bit input: Dynamic clock inversion input
      CLKIN => '0',               			-- 1-bit input: Clock delay input
      CNTVALUEIN => clk_cntvaluein,   		-- 5-bit input: Counter value input
      DATAIN => '0',              			-- 1-bit input: Internal delay data input
      IDATAIN => clk_ab_l,	      			-- 1-bit input: Data input from the I/O
      INC => '0',                 			-- 1-bit input: Increment / Decrement tap delay input
      ODATAIN => '0',             			-- 1-bit input: Output delay data input
      RST => delay_update,        			-- 1-bit input: Active-high reset tap-delay input
      T => '1'                    			-- 1-bit input: 3-state input
   );
----------------------------------------------------------------------------------------------------
-- MMCM ADC / DAC Clocks
----------------------------------------------------------------------------------------------------
-- mmcm_adac_inst : mmcm_adac
mmcm_adac_inst : mmcm_adac
port map (
  clk_in1 => clk_ab_dly,
  CLK_IN2 => clk_ab_dly,
  CLK_IN_SEL => '0',
  clk_out1  => clk_122_88MHz,
  clk_out2  => clk_245_76MHz,
  clk_out3  => clk_491_52MHz,
  reset     => cpu_reset,
  locked    => mmcm_adac_locked
);

----------------------------------------------------------------------------------------------------
-- Reset sequence
----------------------------------------------------------------------------------------------------
process (mmcm_adac_locked, clk_245_76MHz)
  variable cnt : integer range 0 to 1023 := 0;
begin
  if (mmcm_adac_locked = '0') then
    cnt := 0;
    rst <= '1';
  elsif (rising_edge(clk_245_76MHz)) then
    -- DDC and DUC are kept in reset state for a while...
    if (cnt < 1023) then
      cnt := cnt + 1;
      rst <= '1';
    else
      cnt := cnt;
      rst <= '0';
    end if;
  end if;
end process;	
	 
process (mmcm_adac_locked, clk_122_88MHz)
  variable cnt : integer range 0 to 1023 := 0;
begin
  if (mmcm_adac_locked = '0') then

    cnt := 0;
    frame <= '0';
    txenable <= '0';

  elsif (rising_edge(clk_122_88MHz)) then

    if (cnt < 1023) then
      cnt := cnt + 1;
    else
      cnt := cnt;
    end if;
	 
    -- The iSERDES and OSERDES blocks are synchronously reset for 1 CLKDIV cycle @ 122.88 MHz ...
    if (cnt = 255) then
      io_rst <= '1';
    else
      io_rst <= '0';
    end if;

    -- Then a frame pulse is transmitted to the DAC...
    if (cnt = 511) then
      frame <= '1';
    else
      frame <= '0';
    end if;

    -- Finally the TX enable for the DAC can by pulled high.
    if (cnt = 1023) then
      txenable <= '1';
    end if;

  end if;
end process;

----------------------------------------------------------------------------------------------------
-- Channel A data from ADC
----------------------------------------------------------------------------------------------------
adc_data_a: for i in 0 to 6 generate

  -- Differential input buffer with termination (LVDS)
  ibufds_inst : ibufds
  generic map (
    IOSTANDARD => "LVDS_25",
	 IBUF_LOW_PWR => FALSE,
    DIFF_TERM  => TRUE
  )
  port map (
    i  => cha_p(i),
    ib => cha_n(i),
    o  => cha_ddr(i)
  );

   IDELAYE1_inst_ADC_ch_A : IODELAYE1
   generic map (
      CINVCTRL_SEL => FALSE,          -- Enable dynamic clock inversion (TRUE/FALSE)
      DELAY_SRC => "I",               -- Delay input ("I", "CLKIN", "DATAIN", "IO", "O")
      HIGH_PERFORMANCE_MODE => TRUE,  -- Reduced jitter (TRUE), Reduced power (FALSE)
      IDELAY_TYPE => "VAR_LOADABLE",  -- "DEFAULT", "FIXED", "VARIABLE", or "VAR_LOADABLE" 
      IDELAY_VALUE => CHA_IDELAY,     -- Input delay tap setting (0-31)
      ODELAY_TYPE => "FIXED",         -- "FIXED", "VARIABLE", or "VAR_LOADABLE" 
      ODELAY_VALUE => 0,              -- Output delay tap setting (0-31)
      REFCLK_FREQUENCY => 200.00,     -- IDELAYCTRL clock input frequency in MHz
      SIGNAL_PATTERN => "DATA"        -- "DATA" or "CLOCK" input signal
   )
   port map (
      CNTVALUEOUT => cha_cntvalueout(i), 	-- 5-bit output: Counter value output	-- NOTE: need clock-domain crossing 
      DATAOUT => cha_ddr_dly(i),         	-- 1-bit output: Delayed data output
      C => clk_122_88MHz,                 -- 1-bit input: Clock input				-- NOTE: Virtex-6 IODELAYE1 imposes same clock source as iSERDES CLKDIV
      CE => '0',                  			-- 1-bit input: Active high enable increment/decrement input
      CINVCTRL => '0',            			-- 1-bit input: Dynamic clock inversion input
      CLKIN => '0',               			-- 1-bit input: Clock delay input
      CNTVALUEIN => cha_cntvaluein,   		-- 5-bit input: Counter value input
      DATAIN => '0',              			-- 1-bit input: Internal delay data input
      IDATAIN => cha_ddr(i),      			-- 1-bit input: Data input from the I/O
      INC => '0',                 			-- 1-bit input: Increment / Decrement tap delay input
      ODATAIN => '0',             			-- 1-bit input: Output delay data input
      RST => delay_update,        			-- 1-bit input: Active-high reset tap-delay input
      T => '1'                    			-- 1-bit input: 3-state input
   );

   ISERDESE1_adc_cha : ISERDESE1
   generic map (
      DATA_RATE => "DDR",         			-- SDR or DDR
      DATA_WIDTH => 4,            			-- Parallel data width (2-8, 10)
      DYN_CLKDIV_INV_EN => TRUE, 			-- Enable DYNCLKDIVINVSEL inversion (TRUE/FALSE)
      DYN_CLK_INV_EN => FALSE,    			-- Enable DYNCLKINVSEL inversion (TRUE/FALSE)
      -- INIT_Q1 - INIT_Q4: Initial value on the Q outputs (0/1)
      INIT_Q1 => '0',
      INIT_Q2 => '0',
      INIT_Q3 => '0',
      INIT_Q4 => '0',
      INTERFACE_TYPE => "NETWORKING", 		-- MEMORY, MEMORY_DDR3, MEMORY_QDR, NETWORKING, or OVERSAMPLE
      IOBDELAY => "IFD",          			-- NONE, IBUF, IFD, BOTH
														-- *** "IFD" routes the delayed path from IO through IDELAY into the ISERDES
      NUM_CE => 1,                			-- Number of clock enables (1 or 2)
      OFB_USED => FALSE,          			-- Select OFB path (TRUE/FALSE)
      SERDES_MODE => "MASTER",    			-- MASTER or SLAVE
      -- SRVAL_Q1 - SRVAL_Q4: Q output values when SR is used (0/1)
      SRVAL_Q1 => '0',
      SRVAL_Q2 => '0'
--      SRVAL_Q3 => '0',
--      SRVAL_Q4 => '0' 
   )
   port map (
--      O => O,                       		-- 1-bit Combinatorial output
      -- Q1 - Q6: 1-bit (each) Data outputs
		Q1 	=> cha_sdr_sample1(2*i + 1),
		Q2 	=> cha_sdr_sample1(2*i),
		Q3 	=> cha_sdr_sample0(2*i + 1),
		Q4 	=> cha_sdr_sample0(2*i),
--		Q5 	=>
--		Q6 	=>
      -- SHIFTOUT1/SHIFTOUT2: 1-bit (each) Cascade outputs for data width expansion
--      SHIFTOUT1 => SHIFTOUT1,
--      SHIFTOUT2 => SHIFTOUT2,
		BITSLIP => '0',				-- 1-bit The BITSLIP pin performs a Bitslip operation synchronous to CLKDIV
											-- when asserted (active High). Subsequently, the data seen on the Q1 to Q6
											-- output ports will shift, as in a barrel-shifter operation, one position
											-- every time Bitslip is invoked (DDR operation is different from SDR).
      -- CE1, CE2: 1-bit (each) Data register clock enable inputs
      CE1 => '1',
      CE2 => '0',
      CLK => clk_245_76MHz,				-- 1-bit Clock input
		CLKB => not clk_245_76MHz,		-- 1-bit Secondary clock input
      CLKDIV => clk_122_88MHz,	-- 1-bit Divided clock input
--      D => adc_data_bus_ddr(n),			-- 1-bit Data input (this path is selected if IOBDELAY => "IBUF" or "NONE")
      D => '0',
      DDLY => cha_ddr_dly(i),					-- 1-bit Serial input data from IODELAY (this path is selected if IOBDELAY => "IFD")
--		  DDLY => '0',
      DYNCLKDIVSEL => '1', 					-- 1-bit Dynamic CLKDIV inversion input
      DYNCLKSEL => '0',       				-- 1-bit Dynamic CLK/CLKB inversion input
      OCLK => '0',                 			-- 1-bit High speed output clock input
      OFB => '0',                   		-- 1-bit Feedback path input
      RST => io_rst,                   	-- 1-bit Active high asynchronous reset input
      -- SHIFTIN1/SHIFTIN2: 1-bit (each) Cascade inputs for data width expansion
      SHIFTIN1 => '0',
      SHIFTIN2 => '0' 
   );
end generate;

----------------------------------------------------------------------------------------------------
-- Channel B data from ADC
----------------------------------------------------------------------------------------------------
adc_data_b: for i in 0 to 6 generate

  -- Differantial input buffer with termination (LVDS)
  ibufds_inst : ibufds
  generic map (
    IOSTANDARD => "LVDS_25",
	 IBUF_LOW_PWR => FALSE,
    DIFF_TERM  => TRUE
  )
  port map (
    i  => chb_p(i),
    ib => chb_n(i),
    o  => chb_ddr(i)
  );

   IDELAYE1_inst_ADC_ch_B : IODELAYE1
   generic map (
      CINVCTRL_SEL => FALSE,          -- Enable dynamic clock inversion (TRUE/FALSE)
      DELAY_SRC => "I",               -- Delay input ("I", "CLKIN", "DATAIN", "IO", "O")
      HIGH_PERFORMANCE_MODE => TRUE,  -- Reduced jitter (TRUE), Reduced power (FALSE)
      IDELAY_TYPE => "VAR_LOADABLE",  -- "DEFAULT", "FIXED", "VARIABLE", or "VAR_LOADABLE" 
      IDELAY_VALUE => CHB_IDELAY,     -- Input delay tap setting (0-31)
      ODELAY_TYPE => "FIXED",         -- "FIXED", "VARIABLE", or "VAR_LOADABLE" 
      ODELAY_VALUE => 0,              -- Output delay tap setting (0-31)
      REFCLK_FREQUENCY => 200.00,     -- IDELAYCTRL clock input frequency in MHz
      SIGNAL_PATTERN => "DATA"        -- "DATA" or "CLOCK" input signal
   )
   port map (
      CNTVALUEOUT => chb_cntvalueout(i), 	-- 5-bit output: Counter value output	-- NOTE: need clock-domain crossing 
      DATAOUT => chb_ddr_dly(i),         	-- 1-bit output: Delayed data output
      C => clk_122_88MHz,                 -- 1-bit input: Clock input				-- NOTE: Virtex-6 IODELAYE1 imposes same clock source as iSERDES CLKDIV
      CE => '0',                  			-- 1-bit input: Active high enable increment/decrement input
      CINVCTRL => '0',            			-- 1-bit input: Dynamic clock inversion input
      CLKIN => '0',               			-- 1-bit input: Clock delay input
      CNTVALUEIN => chb_cntvaluein,   		-- 5-bit input: Counter value input
      DATAIN => '0',              			-- 1-bit input: Internal delay data input
      IDATAIN => chb_ddr(i),      			-- 1-bit input: Data input from the I/O
      INC => '0',                 			-- 1-bit input: Increment / Decrement tap delay input
      ODATAIN => '0',             			-- 1-bit input: Output delay data input
      RST => delay_update,        			-- 1-bit input: Active-high reset tap-delay input
      T => '1'                    			-- 1-bit input: 3-state input
   );

  ISERDESE1_adc_chb : ISERDESE1
   generic map (
      DATA_RATE => "DDR",         			-- SDR or DDR
      DATA_WIDTH => 4,            			-- Parallel data width (2-8, 10)
      DYN_CLKDIV_INV_EN => TRUE, 			-- Enable DYNCLKDIVINVSEL inversion (TRUE/FALSE)
      DYN_CLK_INV_EN => FALSE,    			-- Enable DYNCLKINVSEL inversion (TRUE/FALSE)
      -- INIT_Q1 - INIT_Q4: Initial value on the Q outputs (0/1)
      INIT_Q1 => '0',
      INIT_Q2 => '0',
      INIT_Q3 => '0',
      INIT_Q4 => '0',
      INTERFACE_TYPE => "NETWORKING", 		-- MEMORY, MEMORY_DDR3, MEMORY_QDR, NETWORKING, or OVERSAMPLE
      IOBDELAY => "IFD",          			-- NONE, IBUF, IFD, BOTH
														-- *** "IFD" routes the delayed path from IO through IDELAY into the ISERDES
      NUM_CE => 1,                			-- Number of clock enables (1 or 2)
      OFB_USED => FALSE,          			-- Select OFB path (TRUE/FALSE)
      SERDES_MODE => "MASTER",    			-- MASTER or SLAVE
      -- SRVAL_Q1 - SRVAL_Q4: Q output values when SR is used (0/1)
      SRVAL_Q1 => '0',
      SRVAL_Q2 => '0'
--      SRVAL_Q3 => '0',
--      SRVAL_Q4 => '0' 
   )
   port map (
--      O => O,                       		-- 1-bit Combinatorial output
      -- Q1 - Q6: 1-bit (each) Data outputs
		Q1 	=> chb_sdr_sample1(2*i + 1),
		Q2 	=> chb_sdr_sample1(2*i),
		Q3 	=> chb_sdr_sample0(2*i + 1),
		Q4 	=> chb_sdr_sample0(2*i),
--		Q5 	=>
--		Q6 	=>
      -- SHIFTOUT1/SHIFTOUT2: 1-bit (each) Cascade outputs for data width expansion
--      SHIFTOUT1 => SHIFTOUT1,
--      SHIFTOUT2 => SHIFTOUT2,
		BITSLIP => '0',				-- 1-bit The BITSLIP pin performs a Bitslip operation synchronous to CLKDIV
											-- when asserted (active High). Subsequently, the data seen on the Q1 to Q6
											-- output ports will shift, as in a barrel-shifter operation, one position
											-- every time Bitslip is invoked (DDR operation is different from SDR).
      -- CE1, CE2: 1-bit (each) Data register clock enable inputs
      CE1 => '1',
      CE2 => '0',
      CLK => clk_245_76MHz,				-- 1-bit Clock input
		CLKB => not clk_245_76MHz,		-- 1-bit Secondary clock input
      CLKDIV => clk_122_88MHz,	-- 1-bit Divided clock input
--      D => adc_data_bus_ddr(n),			-- 1-bit Data input (this path is selected if IOBDELAY => "IBUF" or "NONE")
      D => '0',
      DDLY => chb_ddr_dly(i),					-- 1-bit Serial input data from IODELAY (this path is selected if IOBDELAY => "IFD")
--		  DDLY => '0',
      DYNCLKDIVSEL => '1', 					-- 1-bit Dynamic CLKDIV inversion input
      DYNCLKSEL => '0',       				-- 1-bit Dynamic CLK/CLKB inversion input
      OCLK => '0',                 			-- 1-bit High speed output clock input
      OFB => '0',                   		-- 1-bit Feedback path input
      RST => io_rst,                   	-- 1-bit Active high asynchronous reset input
      -- SHIFTIN1/SHIFTIN2: 1-bit (each) Cascade inputs for data width expansion
      SHIFTIN1 => '0',
      SHIFTIN2 => '0' 
   );
end generate;

----------------------------------------------------------------------------------------------------
-- Re-clock dual 122.88 MSPS aligned output streams from iSERDES to 245.76 MHz clock domain
----------------------------------------------------------------------------------------------------
	process (clk_245_76MHz)
	begin
		if falling_edge(clk_245_76MHz) then
			if (mmcm_adac_locked = '0') then
				adc_dout_122_88_MSPS_valid <= '0';
			else
				adc_dout_122_88_MSPS_valid <= not adc_dout_122_88_MSPS_valid; -- make valid pulse to accompany 2 aligned ADC sample streams decimated by 2 @ 122.88 MSPS
			end if;
		end if;
	end process;
	
	process (clk_245_76MHz)
	begin
		if rising_edge(clk_245_76MHz) then
			if adc_dout_122_88_MSPS_valid = '1' then
				cha_sdr_sample0_re_clocked <= cha_sdr_sample0;
				cha_sdr_sample1_re_clocked <= cha_sdr_sample1;
				chb_sdr_sample0_re_clocked <= chb_sdr_sample0;
				chb_sdr_sample1_re_clocked <= chb_sdr_sample1;
			end if;
		end if;
	end process;

----------------------------------------------------------------------------------------------------
-- Re-mux to 245.76 MHz the 2 aligned ADC sample streams decimated by 2 @ 122.88 MSPS at output of ISERDES
-- to provide to the user a choice of:
-- -- re-mux'd 245.76 MSPS signals
--    or
-- -- 2 aligned sample streams decimated by 2 @ 122.88 MSPS
-- -- ... ideal for custom polyhase 2X decimation filter
----------------------------------------------------------------------------------------------------
	process (clk_245_76MHz)
	begin
		if rising_edge(clk_245_76MHz) then
			if (mmcm_adac_locked = '1') then
				adc_dout_valid <= '1';  -- make valid pulse to accompany re-mux'd 245.76 MSPS ADC samples
				if (clk_245_76MHz_count = '0') then
					if adc_cha_re_mux_polarity = '0' then
						adc_dout_a <= cha_sdr_sample0;
					else
						adc_dout_a <= cha_sdr_sample1;
					end if;
					if adc_chb_re_mux_polarity = '0' then
						adc_dout_b <= chb_sdr_sample0;
					else
						adc_dout_b <= chb_sdr_sample1;
					end if;
					clk_245_76MHz_count <= '1';
				elsif (clk_245_76MHz_count = '1') then
					if adc_cha_re_mux_polarity = '0' then
						adc_dout_a <= cha_sdr_sample1;
					else
						adc_dout_a <= cha_sdr_sample0;
					end if;
					if adc_chb_re_mux_polarity = '0' then
						adc_dout_b <= chb_sdr_sample1;
					else
						adc_dout_b <= chb_sdr_sample0;
					end if;
					clk_245_76MHz_count <= '0';
				end if;
			else
				clk_245_76MHz_count <= '0';
				adc_dout_valid <= '0';  -- make valid pulse to accompany re-mux'd 245.76 MSPS ADC samples
			end if;
		end if;
	end process;
   
	
----------------------------------------------------------------------------------
-- ILA for monitor of ADC calibration
----------------------------------------------------------------------------------

ILA_ADC_cali_inst : ILA_ADC_cali
  port map (
    CONTROL => ila_adc_cali_control,
    CLK => clk_245_76Mhz,
    DATA => ILA_ADC_cali_data,  
    TRIG0 => adc_dout_b,
    TRIG1 => ILA_ADC_cali_inst_trig1,
    TRIG2 => ILA_ADC_cali_inst_trig2);
	 
	 ILA_ADC_cali_inst_trig1(0) <= ADC_chb_trace_edge;
	 ILA_ADC_cali_inst_trig2(0) <= delay_update;
	 
	 ILA_ADC_cali_data <= adc_dout_b & ADC_chb_calibration_state & chb_cntvaluein & delay_update & adc_chb_re_mux_polarity;

----------------------------------------------------------------------------------
-- ADC calibration Channel B
----------------------------------------------------------------------------------

ADC_auto_calibration_chB: ADC_auto_calibration
 generic map(
		  MAX_PATTERN_CNT => MAX_PATTERN_CNT,
		  INIT_IDELAY => CHB_IDELAY
		)
 Port map( 
        reset => arst,
		  clk => clk_245_76Mhz,
		  ADC_calibration_start => ADC_chb_calibration_start,
		  ADC_data => adc_dout_b,
		  re_mux_polarity => adc_chb_re_mux_polarity,
		  trace_edge => ADC_chb_trace_edge,
		  ADC_calibration_state => ADC_chb_calibration_state,
		  iDelay_cnt => chb_cntvaluein_update_245_76MHz,
		  ADC_calibration_done => ADC_chb_calibration_done,
		  ADC_calibration_good => ADC_chb_calibration_good);

----------------------------------------------------------------------------------
-- ADC calibration Channel A
----------------------------------------------------------------------------------

ADC_auto_calibration_chA: ADC_auto_calibration
 generic map(
		  MAX_PATTERN_CNT => MAX_PATTERN_CNT,
		  INIT_IDELAY => CHA_IDELAY
		)
 Port map(
        reset => arst,
		  clk => clk_245_76Mhz,
		  ADC_calibration_start => ADC_cha_calibration_start,
		  ADC_data => adc_dout_a,
		  re_mux_polarity => adc_cha_re_mux_polarity,
		  trace_edge => ADC_cha_trace_edge,
		  ADC_calibration_state => ADC_cha_calibration_state,
		  iDelay_cnt => cha_cntvaluein_update_245_76MHz,
		  ADC_calibration_done => ADC_cha_calibration_done,
		  ADC_calibration_good => ADC_cha_calibration_good);

----------------------------------------------------------------------------------------------------
-- DAC Reference Clock Input
----------------------------------------------------------------------------------------------------
-- clk_to_fpga_p/n is an additional LVDS clock from the CDCE72010, but currently not used in this
-- reference design. This additional clock might be used in applications where the ADC is not used
-- and a reference clock for the DAC is required.
----------------------------------------------------------------------------------------------------
ibufds_ref_clk : ibufds
generic map (
  IOSTANDARD => "LVDS_25",
  DIFF_TERM => TRUE
)
port map (
  i  => clk_to_fpga_p,
  ib => clk_to_fpga_n,
  o  => clk_to_fpga
);

----------------------------------------------------------------------------------------------------
-- Output serdes and LVDS buffer for DAC clock
----------------------------------------------------------------------------------------------------
oserdes_clock : oserdes
generic map (
  DATA_RATE_OQ => "DDR",
  DATA_RATE_TQ => "DDR",
  DATA_WIDTH => 4,
  INIT_OQ => '0',
  INIT_TQ => '0',
  SERDES_MODE => "MASTER",
  SRVAL_OQ => '0',
  SRVAL_TQ => '0',
  TRISTATE_WIDTH => 1
)
port map (
  oq        => dac_dclk_prebuf,
  shiftout1 => open,
  shiftout2 => open,
  tq        => open,
  clk       => clk_491_52MHz,
  clkdiv    => clk_245_76MHz,
  d1        => '1',
  d2        => '0',
  d3        => '1',
  d4        => '0',
  d5        => '0',
  d6        => '0',
  oce       => '1',
  rev       => '0',
  shiftin1  => '0',
  shiftin2  => '0',
  sr        => io_rst,
  t1        => '0',
  t2        => '0',
  t3        => '0',
  t4        => '0',
  tce       => '0'
);

--output buffer
obufds_clock : obufds_lvds_25
port map (
  i  => dac_dclk_prebuf,
  o  => dac_dclk_p,
  ob => dac_dclk_n
);

----------------------------------------------------------------------------------------------------
-- Output serdes and LVDS buffers for DAC data
----------------------------------------------------------------------------------------------------
dac_data: for i in 0 to 7 generate

  --oserdes in data path
  oserdes_data : oserdes
  generic map (
    DATA_RATE_OQ => "DDR",
    DATA_RATE_TQ => "DDR",
    DATA_WIDTH => 4,
    INIT_OQ => '0',
    INIT_TQ => '0',
    SERDES_MODE => "MASTER",
    SRVAL_OQ => '0',
    SRVAL_TQ => '0',
    TRISTATE_WIDTH => 1
  )
  port map (
    oq        => dac_data_prebuf(i),
    shiftout1 => open,
    shiftout2 => open,
    tq        => open,
    clk       => clk_491_52MHz,
    clkdiv    => clk_245_76MHz,
    d1        => dac_din_a(i + 8),
    d2        => dac_din_a(i),
    d3        => dac_din_b(i + 8),
    d4        => dac_din_b(i),
    d5        => '0',
    d6        => '0',
    oce       => '1',
    rev       => '0',
    shiftin1  => '0',
    shiftin2  => '0',
    sr        => io_rst,
    t1        => '0',
    t2        => '0',
    t3        => '0',
    t4        => '0',
    tce       => '0'
  );

  --output buffers
  obufds_data : obufds_lvds_25
  port map (
    i  => dac_data_prebuf(i),
    o  => dac_data_p(i),
    ob => dac_data_n(i)
  );

end generate;

----------------------------------------------------------------------------------------------------
-- Output serdes and LVDS buffer for DAC frame
----------------------------------------------------------------------------------------------------
oserdes_frame : oserdes
generic map (
  DATA_RATE_OQ => "DDR",
  DATA_RATE_TQ => "DDR",
  DATA_WIDTH => 4,
  INIT_OQ => '0',
  INIT_TQ => '0',
  SERDES_MODE => "MASTER",
  SRVAL_OQ => '0',
  SRVAL_TQ => '0',
  TRISTATE_WIDTH => 1
)
port map (
  oq        => dac_frame_prebuf,
  shiftout1 => open,
  shiftout2 => open,
  tq        => open,
  clk       => clk_491_52MHz,
  clkdiv    => clk_245_76MHz,
  d1        => frame,
  d2        => frame,
  d3        => frame,
  d4        => frame,
  d5        => '0',
  d6        => '0',
  oce       => '1',
  rev       => '0',
  shiftin1  => '0',
  shiftin2  => '0',
  sr        => io_rst,
  t1        => '0',
  t2        => '0',
  t3        => '0',
  t4        => '0',
  tce       => '0'
);

--output buffer
obufds_frame : obufds_lvds_25
port map (
  i  => dac_frame_prebuf,
  o  => dac_frame_p,
  ob => dac_frame_n
);

----------------------------------------------------------------------------------------------------
-- Configuring the FMC150 card
----------------------------------------------------------------------------------------------------
-- the fmc150_spi_ctrl component configures the devices on the FMC150 card through the Serial
-- Peripheral Interfaces (SPI) and some additional direct control signals.
----------------------------------------------------------------------------------------------------
fmc150_spi_ctrl_inst : fmc150_spi_ctrl
port map (
  odata           => odata,

  addr            => addr,
  idata           => idata,
  rd_n_wr         => rd_n_wr,
  cdce72010_valid => cdce72010_valid,
  ads62p49_valid  => ads62p49_valid,
  dac3283_valid   => dac3283_valid,
  amc7823_valid   => amc7823_valid,

  busy            => busy,

  external_clock  => external_clock,

  rst             => arst,
  clk             => clk_100MHz,
  spi_sclk        => spi_sclk,
  spi_sdata       => spi_sdata,
  adc_n_en        => adc_n_en,
  adc_sdo         => adc_sdo,
  adc_reset       => adc_reset,
  cdce_n_en       => cdce_n_en,
  cdce_sdo        => cdce_sdo,
  cdce_n_reset    => cdce_n_reset,
  cdce_n_pd       => cdce_n_pd,
  ref_en          => ref_en,
  pll_status      => pll_status,
  dac_n_en        => dac_n_en,
  dac_sdo         => dac_sdo,
  mon_n_en        => mon_n_en,
  mon_sdo         => mon_sdo,
  mon_n_reset     => mon_n_reset,
  mon_n_int       => mon_n_int,
  prsnt_m2c_l     => prsnt_m2c_l,
  init_amc7823_done => fmc150_spi_ctrl_done
);

----------------------------------------------------------------------------------------------------
-- Connect entity
----------------------------------------------------------------------------------------------------

gpio_led(0) <= gpio_dip_sw(0);
gpio_led(1) <= gpio_dip_sw(1);
gpio_led(2) <= gpio_dip_sw(2);
gpio_led(3) <= gpio_dip_sw(3);
gpio_led(4) <= pll_status;
gpio_led(5) <= mmcm_adac_locked;
gpio_led(6) <= mmcm_locked;
gpio_led(7) <= ADC_calibration_good;

ADC_calibration_good <= ADC_chb_calibration_good AND ADC_cha_calibration_good;

gpio_led_c <= gpio_sw_c;
gpio_led_e <= gpio_sw_e;
gpio_led_n <= gpio_sw_n;
gpio_led_s <= gpio_sw_s;
gpio_led_w <= gpio_sw_w;



----------------------------------------------------------------------------------------------------
-- IDELAYCTRL
----------------------------------------------------------------------------------------------------
idelayctrl_inst : idelayctrl
port map (
  rst    => arst,
  refclk => clk_200Mhz,
  rdy    => open
);


sample_pass_inst : sample_pass  port map (
	clk_fs => clk_245_76MHz,
	clk_gfs => clk_491_52MHz,
	din_a	=> adc_dout_a,
	din_b	=> adc_dout_b,
	dout_a	=> dac_din_a,
	dout_b	=> dac_din_b,
	gpio_dip_sw => gpio_dip_sw(7 downto 0) 
);




----------------------------------------------------------------------------------------------------
-- ICON
----------------------------------------------------------------------------------------------------

icon_inst : icon
port map (
  control0 => vio_control,
  control1 => ila_dac_control,
  control2 => ila_adc_cali_control
);






ila_dac_inst : ila_dac
port map (
  clk     => clk_245_76MHz,
  trig0   => ila_dac_trig0,
  control => ila_dac_control
);

ila_dac_trig0 <= dac_din_b & dac_din_a;

end ML605_fmc150_syn;
